-- Version: v0.5
-- Last modified: 24.07.2010
-- Last modified by: bussdee

local function print(value)
	if DEFAULT_CHAT_FRAME then
		DEFAULT_CHAT_FRAME:AddMessage(value);
	end
	
	if SELECTED_CHAT_FRAME and SELECTED_CHAT_FRAME ~= DEFAULT_CHAT_FRAME then
		SELECTED_CHAT_FRAME:AddMessage(value);
	end
end

local function printf(stringFormat, ...)
	print(string.format(stringFormat, ...));
end

local function parseHyperlink(s)
	local link, type, data, text = string.match(s, "(|H(.+):(.+)|h(.+)|h)");
	return type, data, text, link;
end

------------------

local me = {
	name = "WebLinks",
	version = "v0.5",
	author = "",
	description = "", -- localized
	slashCommand = "/weblinks",
	enabled = true,
	debug = false,
	locale = nil,
};

function me.OnLoad(this)
	if this ~= nil then
		this:RegisterEvent("VARIABLES_LOADED");
	end

	-- load default locale if needed
	if me.locale == nil then
		local filename = string.format("Interface\\Addons\\%s\\Bonus\\weblinkslocale.lua", "XBar");
		dofile(filename);
	end
	
	-- set slash commands
	_G[string.format("SLASH_%s1", me.name)] = me.slashCommand;
	SlashCmdList[me.name] = me.ExecuteSlashCommand;
end

function me.OnEvent(this, event, arg1, arg2)
	if event == "VARIABLES_LOADED" then
		me.description = me.locale.AddOnDescription;
	--	me.RegisterAddOnManager();
		me.HookApi();
	end
end

function me.HookApi()
	local originalClickEventHandler = ChatFrame_OnHyperlinkClick;

	ChatFrame_OnHyperlinkClick = function(self, link, key)
		if me.enabled and key == "LBUTTON" then
			local linkType, linkData, linkText = parseHyperlink(link);

			if linkType == "web" then
				if IsShiftKeyDown() then
					ChatEdit_AddItemLink(link);
				elseif not IsCtrlKeyDown() then
					--linkData = string.gsub(linkData, "http;//", "http://");
					--StaticPopupDialogs["OPEN_WEBROWER"].link = linkData;
					--StaticPopup_Show("OPEN_WEBROWER");
					GC_OpenWebRadio(string.gsub(linkData, "http;//", "http://"))
				end
				
				return;
			end
		end

		originalClickEventHandler(self, link, key);
	end;

	local originalAddMessage = DEFAULT_CHAT_FRAME.AddMessage;

	DEFAULT_CHAT_FRAME.AddMessage = function(self, txt, r, g, b)
		if me.enabled then
			txt = me.ConvertWebLinks(txt);
		end

		originalAddMessage(self, txt, r, g, b);
	end;

	-- hook other ChatFrames (thanks to tasquer)
	for i = 3, 10 do
		local frame = getglobal('ChatFrame' .. tostring(i))

		if frame then
			local orig = frame.AddMessage;
			
			frame.AddMessage = function(self, txt, r, g, b)
				if me.enabled then
					txt = me.ConvertWebLinks(txt);
				end

				orig(self, txt, r, g, b);
			end;
		end
	end
end

function me.ConvertWebLinks(txt)
	if txt == nil then return end;
	local newTxt, count = string.gsub(txt, "(http://[^%s]+)", "|Hweb:%1|h|cff30ffff[%1]|r|h");

	if count > 0 then
		txt = string.gsub(newTxt, "web:http://", "web:http;//");
	end

	return txt;
end

function me.Enable(enable)
	me.enabled = enable;
end

function me.ExecuteSlashCommand(frame, txt)
	if txt == nil then
		txt = "";
	else
		txt = string.lower(txt);
	end

	if txt == "" then
		-- toggle
		if me.enabled then
			me.Enable(false);
			printf(me.locale.AddOnDisabled, me.name);
		else
			me.Enable(true);
			printf(me.locale.AddOnEnabled, me.name);
		end
	elseif txt == "on" or txt == "enable" then
		me.Enable(true);
		printf(me.locale.AddOnEnabled, me.name);
	elseif txt == "off" or txt == "disable" then
		me.Enable(false);
		printf(me.locale.AddOnDisabled, me.name);
	end
end

--function me.RegisterAddOnManager()
--	if AddonManager then
--		local addon = {
--			name = me.name,
--			version = me.version,
--			author = me.author,
--			description = me.description,
--			slashCommand = me.slashCommand,
--			icon = nil,
--			category = nil,
--			configFrame = nil,
--			miniButton = nil,
--			disableScript = function() me.Enable(false) end,
--			enableScript = function() me.Enable(true) end,
--		};
--
--		if AddonManager.RegisterAddonTable then
--			AddonManager.RegisterAddonTable(addon);
--		else
--			AddonManager.RegisterAddon(addon.name, addon.description, addon.icon, addon.category, addon.configFrame, addon.slashCommand, addon.miniButton, addon.version, addon.author);
--		end;
--	else
--		printf(me.locale.AddOnLoaded, me.name, me.version);
--	end
--end

_G[me.name] = me;


