----------------------------------------------------------------------------------------------------------
--
--  eCooldownPulse
--
-- written by Elgor
-- released under the Creative Commons License By-Nc-Sa: http://creativecommons.org/licenses/by-nc-sa/3.0/
--
-- release notes
-- ver. 0.1.0 : initial release(alpha-state)
-- ver. 0.1.1 : beta-state release
-- ver. 0.2.0 : added queueing feature
--			    added GUI config
--				added sound function
----------------------------------------------------------------------------------------------------------
--========================================================================================================
-- Addon namespace definition
--========================================================================================================
local eCooldownPulse = {};
_G.eCooldownPulse = eCooldownPulse;

--========================================================================================================
-- Slash command
--========================================================================================================
SLASH_ECP1 = "/ecp";
SlashCmdList["ECP"] = function(editBox, msg)
	eCooldownPulse.CmdHdr(msg);
end

--========================================================================================================
-- Global vars
--========================================================================================================
eCP_Config_Save = {};

--========================================================================================================
-- Local vars
--========================================================================================================
----------
-- Configs
----------
local onConfig = false;
local tempConfig = {};
local defaultConfig = {
	["version"] =	020,
	["init"] =		false,
	["enable"] =	true,
	["pos_x"] =		300,
	["pos_y"] = 	200,
	["size_max"] =	4,
	["show_key"] =	false,
	["sound"] =		"off",
};
local orig_w = 0;
local orig_h = 0;
local ecp_step = 0;
local scalestep = {
	[1]  = "2",
	[2]  = "4",
	[3]  = "6",
	[4]  = "8",
	[5]  = "10",
	[6]  = "12",
	[7]  = "14",
	[8]  = "16",
};
local sound_path = "";

------------------------------
-- Queue table for expired CDs
------------------------------
local CDqueue = {
--	[index] = 	{button_id, 	duration, 		remaining, 		IsSkill},
	[1] = 		{["bid"] = 0, 	["dur"] = 0, 	["rem"] = 0,	["iss"] = false},
	[2] = 		{["bid"] = 0, 	["dur"] = 0, 	["rem"] = 0,	["iss"] = false},
	[3] = 		{["bid"] = 0, 	["dur"] = 0, 	["rem"] = 0,	["iss"] = false},
	[4] = 		{["bid"] = 0, 	["dur"] = 0, 	["rem"] = 0,	["iss"] = false},
	[5] = 		{["bid"] = 0, 	["dur"] = 0, 	["rem"] = 0,	["iss"] = false},
	[6] = 		{["bid"] = 0, 	["dur"] = 0, 	["rem"] = 0,	["iss"] = false},
	[7] = 		{["bid"] = 0, 	["dur"] = 0, 	["rem"] = 0,	["iss"] = false},
	[8] = 		{["bid"] = 0, 	["dur"] = 0, 	["rem"] = 0,	["iss"] = false},
	[9] = 		{["bid"] = 0, 	["dur"] = 0, 	["rem"] = 0,	["iss"] = false},
	[10] = 		{["bid"] = 0, 	["dur"] = 0, 	["rem"] = 0,	["iss"] = false},
};

----------
-- Strings
----------
local AddonName = 			"eCooldownPulse";
local AddonVer =			"v0.2.0";
local DescConfigButton =	"|cffFFFF00Config Menu|r";
local SoundPath1 =			"sound\\interface\\sys_mission_onditions.mp3";
local SoundPath2 =			"sound\\interface\\add_rune.mp3";
local SoundPath3 =			"sound\\interface\\refine_fail.mp3";
local SoundPath4 =			"sound\\interface\\gather_wood_success.mp3";

---------
-- Others
---------
init_done = false;

--------------------------------------------
-- Local references to API for faster access
--------------------------------------------
local _G				= 	_G;
local GetTime 			=	GetTime;
local GetActionCooldown =	GetActionCooldown;
local GetActionInfo     =	GetActionInfo;
local GetUIScale		=	GetUIScale;
local GetSkillDetail	=	GetSkillDetail;
local GetBindingKey		=	GetBindingKey;
local GetNumSkill		=	GetNumSkill;
local PlaySoundByPath	=	PlaySoundByPath;
local tonumber          =	tonumber;
local pairs				=	pairs;
local math_abs			=	math.abs;
local string_sub		=	string.sub;
local string_find		=	string.find;

--========================================================================================================
-- Functions
--========================================================================================================
--------
-- Print
--------
local function DCFmsg(str, r, g, b)
	local red, green, blue;
	if( r == nil or g == nil or b == nil ) then
		red = 1;
		green = 1;
		blue = 0;
	else
		red = r;
		green = g;
		blue = b;
	end
	_G["DEFAULT_CHAT_FRAME"]:AddMessage(str, red, green, blue);
end

--------------------------------
-- OnLoad event(init main frame)
--------------------------------
function eCooldownPulse.OnLoad(this)
	this:RegisterEvent("VARIABLES_LOADED");
	this.lastUpdate = 0;
	this.cdanim = false;
	orig_w, orig_h = this:GetSize();
end

----------------
-- Event Handler
----------------
function eCooldownPulse.OnEvent(this, event)
	if( event == "VARIABLES_LOADED" ) then
		eCooldownPulse.LoadConfig();
		if( tempConfig.version < 20 ) then		-- not compatible config data
			eCooldownPulse.ResetConfig();
			eCooldownPulse.LoadConfig();		-- reload config
			onConfig = true;
		end
		eCooldownPulse.InitFrame();
		eCooldownPulse.SaveConfig();
		init_done = true;
		local name = 	"|cffFFFF00"..AddonName.."|r";
		local ver =		"|cffFFFF00"..AddonVer.."|r";
		local slash =	"|cffFFFF00"..SLASH_ECP1.."|r";
		--DCFmsg("["..name.." "..ver.."] loaded. Use "..slash.." for Configuration.", 0, 1, 0);
	end
end

--------------------
-- Initialize frames
--------------------
function eCooldownPulse.InitFrame()
	local scale = GetUIScale();
	local mainFrame = _G["eCP_Frame"];
	local keyFrame  = _G["eCP_FrameKey"];
	local iconFrame = _G["eCP_FrameIcon"];
	local cbFrame   = _G["eCP_ConfigFrame"];
	mainFrame:ClearAllAnchors();
	mainFrame:SetAnchor("TOPLEFT", "TOPLEFT", "UIParent", tempConfig.pos_x/scale, tempConfig.pos_y/scale);
	local w = orig_w * tempConfig.size_max;
	local h = orig_h * tempConfig.size_max;
	mainFrame:SetSize(w, h);
	keyFrame:SetFontSize(h/5);
	if( not onConfig ) then
		mainFrame:SetMouseEnable(false);
		iconFrame:SetTexture("");
		iconFrame:Hide();
		keyFrame:SetText("");
		keyFrame:Hide();
		cbFrame:Hide();
	else
		mainFrame:SetMouseEnable(true);
		local _, _, icon = GetSkillDetail(1,1);
		iconFrame:SetTexture(icon);
		iconFrame:SetScale(tempConfig.size_max);
		iconFrame:Show();
		keyFrame:SetText("size = "..tempConfig.size_max);
		keyFrame:Show();
		cbFrame:Show();
	end
	iconFrame:SetAlpha(0.5);
	ecp_step = tempConfig.size_max/32;
	if( tempConfig.sound == "off" ) then
		sound_path = "";
	elseif( tempConfig.sound == "type 1" ) then
		sound_path = SoundPath1;
	elseif( tempConfig.sound == "type 2" ) then
		sound_path = SoundPath2;
	elseif( tempConfig.sound == "type 3" ) then
		sound_path = SoundPath3;
	elseif( tempConfig.sound == "type 4" ) then
		sound_path = SoundPath4;
	end
end

-------------------
-- OnUpdate handler
-------------------
function eCooldownPulse.OnUpdate(this)
	if( onConfig or not tempConfig.enable or not init_done ) then
		return;
	end

	if( (GetTime() - this.lastUpdate) < 0.05 ) then
		return;
	else
		this.lastUpdate = GetTime();
	end

	for button_id = 1, ACTIONBAR_NUM_BUTTONS * 4 do
		local duration, remaining = GetActionCooldown(button_id);
		if( duration > 5 and remaining > 0 and remaining < 0.5 ) then
			local IsShared, IsSkill = eCooldownPulse.IsSharedCD(button_id, duration, remaining);
			if( not IsShared ) then
				eCooldownPulse.QueueIn(button_id, duration, remaining, IsSkill);
			end
		end
	end

	if( not this.cdanim ) then
		local button_id = eCooldownPulse.GetBIDfromQueue();
		if( button_id ~= nil ) then
			local icon = GetActionInfo(button_id);
			this.cdanim = true;
			_G["eCP_FrameIcon"]:SetAlpha(0.3);
			_G["eCP_FrameIcon"]:SetScale(tempConfig.size_max/4);
			_G["eCP_FrameIcon"]:SetTexture(icon);
			_G["eCP_FrameIcon"]:Show();
			if( tempConfig.show_key ) then
				local key = eCooldownPulse.GetKeyDef(button_id);
				_G["eCP_FrameKey"]:SetFontSize(orig_h/4);
				_G["eCP_FrameKey"]:SetText(key);
				_G["eCP_FrameKey"]:Show();
			end
			if( tempConfig.sound ~= "off" ) then
				PlaySoundByPath(sound_path);
			end
		end
	else
		local scale = _G["eCP_FrameIcon"]:GetScale();
		local adjuster = 0;
		if( eCooldownPulse.GetNumInQueue() > 1 ) then
			adjuster = 1 - (eCooldownPulse.GetNumInQueue()/10);
		else
			adjuster = 1;
		end
		if( scale < (tempConfig.size_max * adjuster) ) then
			scale = scale + ecp_step;
			_G["eCP_FrameIcon"]:SetScale(scale);
			local alpha = _G["eCP_FrameIcon"]:GetAlpha();
			if( scale < (tempConfig.size_max*0.9) ) then
				alpha = alpha + (ecp_step/2);
			else
				if( alpha > 0.3 ) then
					alpha = alpha - (ecp_step);
				end
			end
			_G["eCP_FrameIcon"]:SetAlpha(alpha);
		else
			this.cdanim = false;
			eCooldownPulse.QueueOut();
			_G["eCP_FrameIcon"]:SetTexture("");
			_G["eCP_FrameIcon"]:Hide();
			_G["eCP_FrameKey"]:SetText("");
			_G["eCP_FrameKey"]:Hide();
		end
	end
end

-------------------------------------------------------------------------
-- Ret1 : true if button_id has shared CD with queued buttons, else false
-- Ret2 : true if button_id is skill, else false
-------------------------------------------------------------------------
function eCooldownPulse.IsSharedCD(button_id, duration, remaining)
	local shared = false;
	local action_name = eCooldownPulse.GetActionName(button_id);
	local IsSkill = eCooldownPulse.ActionIsSkill(button_id, action_name)
	for i = 1, #CDqueue do
		if( CDqueue[i].bid ~= 0 ) then
			local queued_action_name = eCooldownPulse.GetActionName(CDqueue[i].bid);
			if( action_name == queued_action_name ) then
				shared = true;
				break;
			elseif( duration == CDqueue[i].dur and math_abs(remaining - CDqueue[i].rem) < 0.4 ) then
				if( not IsSkill) then
					if( not CDqueue[i].iss ) then
						shared = true;
						break;
					else
						shared = false;
					end
				end
			end
		end
	end
	return shared, IsSkill;
end

----------------------
-- Returns action name
----------------------
function eCooldownPulse.GetActionName(button_id)
	local name, ret;
	_G["eCP_FakeTooltip"]:SetActionItem(button_id);
	_G["eCP_FakeTooltip"]:Hide();
	name = _G["eCP_FakeTooltipTextLeft1"]:GetText();
	if( string_find(name, "+") ) then
		ret = string_sub(name, 1, string_find(name, "+") -1);
	else
		ret = name;
	end
	return ret;
end

-----------------------------------------------------------
-- Retruns true if button_id is a skill, else returns false
-----------------------------------------------------------
function eCooldownPulse.ActionIsSkill(button_id, action_name)
	for tab_id = 1, 5 do
		local numSkills = GetNumSkill(tab_id);
		if( numSkills ) then
			for i = 1, numSkills do
				local skillName = GetSkillDetail(tab_id, i);
				if( skillName == action_name ) then
					return true;
				end
			end
		end
	end
	return false;
end

function eCooldownPulse.InitQueue()
	for i = 1, #CDqueue do
		CDqueue[i].bid = 0;
		CDqueue[i].dur = 0;
		CDqueue[i].rem = 0;
		CDqueue[i].iss = false;
	end
end

---------------------
-- Queue in button_id
---------------------
function eCooldownPulse.QueueIn(button_id, duration, remaining, IsSkill)
	for i = 1, #CDqueue do
		if( CDqueue[i].bid == 0 ) then
			CDqueue[i].bid = button_id;
			CDqueue[i].dur = duration;
			CDqueue[i].rem = remaining;
			CDqueue[i].iss = IsSkill;
--debug
--DCFmsg("QueueIN["..i.."]".." "..tostring(CDqueue[i].bid).." "..
--tostring(CDqueue[i].dur).." "..tostring(CDqueue[i].rem).." "..tostring(CDqueue[i].iss));
--debug
			return;
		end
	end
end

------------------------------------
-- Remove button_id from Queue table
------------------------------------
function eCooldownPulse.QueueOut()
	CDqueue[1].bid = 0;
	CDqueue[1].dur = 0;
	CDqueue[1].rem = 0;
	CDqueue[1].iss = false;
	local num = eCooldownPulse.GetNumInQueue();
	if( num > 0 ) then
		for i = 1, num + 1 do
			if( i == num + 1 ) then
				CDqueue[i].bid = 0;
				CDqueue[i].dur = 0;
				CDqueue[i].rem = 0;
				CDqueue[i].iss = false;
			else
				CDqueue[i].bid = CDqueue[i+1].bid;
				CDqueue[i].dur = CDqueue[i+1].dur;
				CDqueue[i].rem = CDqueue[i+1].rem;
				CDqueue[i].iss = CDqueue[i+1].iss;
			end
		end
	end
--debug
--DCFmsg("queue num = "..eCooldownPulse.GetNumInQueue());
--for i = 1, #CDqueue do
--if( CDqueue[i].bid ~= 0 ) then
--DCFmsg("DUMP Queue["..i.."]".." "..tostring(CDqueue[i].bid).." "..
--tostring(CDqueue[i].dur).." "..tostring(CDqueue[i].rem).." "..tostring(CDqueue[i].iss));
--end
--end
--debug
end

-------------------------------------------------------------------------
-- Returns button_id from queue table, also returns nil if queue is empty
-------------------------------------------------------------------------
function eCooldownPulse.GetBIDfromQueue()
	for i = 1, #CDqueue do
		if( CDqueue[i].bid ~= 0 ) then
			return CDqueue[i].bid;
		end
	end
	return nil;
end

---------------------------------------
-- Returns how many button_ids in Queue
---------------------------------------
function eCooldownPulse.GetNumInQueue()
	local ret = 0;
	for i = 1, #CDqueue do
		if( CDqueue[i].bid ~= 0 ) then
			ret = ret + 1;
		end
	end
	return ret;
end

-----------------------
-- button ID -> key def
-----------------------
function eCooldownPulse.GetKeyDef(button_id)
	local a = 1;
	if( button_id <= 20 ) then
		a = 1;
	elseif( button_id <= 40 ) then
		a = 2;
	elseif( button_id <= 60 ) then
		a = 3;
	elseif( button_id <= 80 ) then
		a = 4;
	end
	local b = button_id - ((a-1)*20);
	local id2name = "ACTIONBAR"..a.."BUTTON"..b;
	local key1, key2 = GetBindingKey(id2name);
	if( key1 == nil ) then
		key1 = "";
	end
	if( key2 == nil ) then
		key2 = "";
	end
	local key = "";
	if( key1 == "" ) then
		key = key2;
	else
		key = key1;
	end
	return key;
end

--------------------------
-- Mouse drag event(start)
--------------------------
function eCooldownPulse.OnMouseDown(this)
	this:StartMoving("TOPLEFT");
end

------------------------
-- Mouse drag event(end)
------------------------
function eCooldownPulse.OnMouseUp(this)
	this:StopMovingOrSizing();
	tempConfig.pos_x, tempConfig.pos_y = this:GetPos();
	eCooldownPulse.SaveConfig();
end

--========================================================================================================
-- Dropdown config menu
--========================================================================================================
--------------------------
-- Mouse over event(start)
--------------------------
function eCooldownPulse.ConfigButtonOnEnter(this)
	local frame = _G["GameTooltip"];
	frame:ClearAllAnchors();
	frame:SetOwner(this, "ANCHOR_TOP", 0, 0);
	frame:ClearLines();
	frame:SetText(DescConfigButton);
	frame:Show();
end

------------------------
-- Mouse over event(end)
------------------------
function eCooldownPulse.ConfigButtonOnLeave(this)
	_G["GameTooltip"]:Hide();
end

----------------
-- OnClick event
----------------
function eCooldownPulse.ConfigButtonOnClick(this)
	ToggleDropDownMenu(_G["eCP_ConfigMenu"]);
end

---------------
-- OnLoad event
---------------
function eCooldownPulse.DropDownMenuOnLoad(this)
	UIDropDownMenu_Initialize(this, eCooldownPulse_OptionMenu, "MENU");
end

--------------
-- Option menu
--------------
function eCooldownPulse_OptionMenu()
	local info = {};
	if( UIDROPDOWNMENU_MENU_LEVEL == 1 ) then
		info = {};
		info.text = "Addon Management";
		info.hasArrow = 1;
		info.notCheckable = 1;
		info.value = 1;
		UIDropDownMenu_AddButton(info, 1);

		info = {};
		info.text = "Scale";
		info.hasArrow = 1;
		info.notCheckable = 1;
		info.value = 2;
		UIDropDownMenu_AddButton(info, 1);

		info = {};
		info.text = "Shortcut Key";
		info.hasArrow = 1;
		info.notCheckable = 1;
		info.value = 3;
		UIDropDownMenu_AddButton(info, 1);

		info = {};
		info.text = "Sound";
		info.hasArrow = 1;
		info.notCheckable = 1;
		info.value = 4;
		UIDropDownMenu_AddButton(info, 1);

		info = {};
		info.text = "Reset Config";
		info.notCheckable = 1;
		info.func = function()
			 eCooldownPulse.ResetConfigOnDemand();
		end
		info.value = 5;
		UIDropDownMenu_AddButton(info, 1);

		info = {};
		info.text = "Close";
		info.notCheckable = 1;
		info.func = function()
			onConfig = false;
			eCooldownPulse.ConfigChanged();
		end
		info.value = 6;
		UIDropDownMenu_AddButton(info, 1);

		info = {};
		info.text = "Cancel";
		info.notCheckable = 1;
		info.func = function()
		end
		info.value = 7;
		UIDropDownMenu_AddButton(info, 1);
	elseif( UIDROPDOWNMENU_MENU_LEVEL == 2 ) then
		if( UIDROPDOWNMENU_MENU_VALUE == 1 ) then
			info = {};
			info.text = "Enable";
			info.func = function()
				tempConfig.enable = true;
				eCooldownPulse.ConfigChanged();
			end
			info.checked = tempConfig.enable;
			info.value = 1;
			UIDropDownMenu_AddButton(info, 2);

			info = {};
			info.text = "Disable";
			info.func = function()
				tempConfig.enable = false;
				eCooldownPulse.ConfigChanged();
			end
			info.checked = not tempConfig.enable;
			info.value = 2;
			UIDropDownMenu_AddButton(info, 2);
		elseif( UIDROPDOWNMENU_MENU_VALUE == 2 ) then
			for i = 1, #scalestep do
				info = {};
				info.text = scalestep[i];
				info.func = function()
					tempConfig.size_max = tonumber(scalestep[i]);
					eCooldownPulse.ConfigChanged();
				end
				if( tempConfig.size_max == tonumber(scalestep[i]) ) then
					info.checked = true;
				else
					info.checked = false;
				end
				info.value = i;
				UIDropDownMenu_AddButton(info, 2);
			end
		elseif( UIDROPDOWNMENU_MENU_VALUE == 3 ) then
			info = {};
			info.text = "Show";
			info.func = function()
				tempConfig.show_key = true;
				eCooldownPulse.ConfigChanged();
			end
			info.checked = tempConfig.show_key;
			info.value = 1;
			UIDropDownMenu_AddButton(info, 2);

			info = {};
			info.text = "Not Show";
			info.func = function()
				tempConfig.show_key = false;
				eCooldownPulse.ConfigChanged();
			end
			info.checked = not tempConfig.show_key;
			info.value = 2;
			UIDropDownMenu_AddButton(info, 2);
		elseif( UIDROPDOWNMENU_MENU_VALUE == 4 ) then
			info = {};
			info.text = "off";
			info.func = function()
				tempConfig.sound = "off";
				eCooldownPulse.ConfigChanged();
			end
			if( tempConfig.sound == "off" ) then
				info.checked = true;
			else
				info.checked = false;
			end
			info.value = 1;
			UIDropDownMenu_AddButton(info, 2);

			info = {};
			info.text = "type 1";
			info.func = function()
				tempConfig.sound = "type 1";
				PlaySoundByPath(SoundPath1);
				eCooldownPulse.ConfigChanged();
			end
			if( tempConfig.sound == "type 1" ) then
				info.checked = true;
			else
				info.checked = false;
			end
			info.value = 2;
			UIDropDownMenu_AddButton(info, 2);

			info = {};
			info.text = "type 2";
			info.func = function()
				tempConfig.sound = "type 2";
				PlaySoundByPath(SoundPath2);
				eCooldownPulse.ConfigChanged();
			end
			if( tempConfig.sound == "type 2" ) then
				info.checked = true;
			else
				info.checked = false;
			end
			info.value = 3;
			UIDropDownMenu_AddButton(info, 2);

			info = {};
			info.text = "type 3";
			info.func = function()
				tempConfig.sound = "type 3";
				PlaySoundByPath(SoundPath3);
				eCooldownPulse.ConfigChanged();
			end
			if( tempConfig.sound == "type 3" ) then
				info.checked = true;
			else
				info.checked = false;
			end
			info.value = 4;
			UIDropDownMenu_AddButton(info, 2);

			info = {};
			info.text = "type 4";
			info.func = function()
				tempConfig.sound = "type 4";
				PlaySoundByPath(SoundPath4);
				eCooldownPulse.ConfigChanged();
			end
			if( tempConfig.sound == "type 4" ) then
				info.checked = true;
			else
				info.checked = false;
			end
			info.value = 5;
			UIDropDownMenu_AddButton(info, 2);
		end
	end
end

-------------------------------------------
-- Reflect and save config via DropDownMenu
-------------------------------------------
function eCooldownPulse.ConfigChanged()
	eCooldownPulse.InitFrame();
	eCooldownPulse.SaveConfig();
end

------------------------
-- Slash command handler
------------------------
function eCooldownPulse.CmdHdr(msg)
	local space = msg:find(" ");
	local p = msg:lower(msg);
	local p1, p2;
	if( space ) then
		p1 = p:sub(1, space - 1);
		p2 = p:sub(space + 1, -1):gsub(" ", "");
	else
		p1 = p;
	end
	if( p1 == "" ) then
		if( onConfig ) then
			onConfig = false;
			eCooldownPulse.InitFrame();
		else
			onConfig = true;
			eCooldownPulse.InitFrame();
		end
	elseif( p1 == "reset" ) then
		eCooldownPulse.ResetConfigOnDemand();
		DCFmsg("eCP:config data -> default", 0, 1, 0);
	else
		DCFmsg("invalid parameter(s)", 1, 0.3, 0);
	end
	eCooldownPulse.SaveConfig();
end

------------------------------
-- Resetting config to default
------------------------------
function eCooldownPulse.ResetConfig()
	-- default data -> save data
	for key, value in pairs(defaultConfig) do
		eCP_Config_Save[key] = value;
	end
	eCP_Config_Save.init = true;
	SaveVariables("eCP_Config_Save");
end

----------------------
-- Loading config data
----------------------
function eCooldownPulse.LoadConfig()
	local need2save = false;
	-- saved data -> internal data
	for key, value in pairs(defaultConfig) do
		if( eCP_Config_Save[key] ~= nil ) then
			tempConfig[key] = eCP_Config_Save[key];
		else
			tempConfig[key] = value;		-- if its a new config, use default one
			need2save = true;
		end
	end
	if( need2save ) then
		eCP_Config_Save.init = true;
		onConfig = true;					-- open GUI for first boot
		SaveVariables("eCP_Config_Save");
	end
end

---------------------
-- Saving config data
---------------------
function eCooldownPulse.SaveConfig()
	-- internal data -> save data
	for key, value in pairs(tempConfig) do
		eCP_Config_Save[key] = value;
	end
	eCP_Config_Save.init = true;
	SaveVariables("eCP_Config_Save");
end
--------------------------------
-- Resetting config from command
--------------------------------
function eCooldownPulse.ResetConfigOnDemand()
	eCooldownPulse.ResetConfig();	-- default data -> save data
	eCooldownPulse.LoadConfig();	-- saved data -> internal data
	eCooldownPulse.InitFrame();		-- reset frame
	eCooldownPulse.InitQueue();		-- reset Queue table
end
